from typing import Optional, Sequence

from faker import Factory
from tabledata import TableData

from ._common import get_providers


class TableFaker:
    def __init__(self, locale: Optional[str] = None, seed: Optional[int] = None) -> None:
        self.__fake = Factory.create(locale)

        if seed is not None:
            self.__fake.seed(seed)

    def generate(
        self,
        providers: Sequence[str],
        rows: int,
        table_name: Optional[str] = None,
        headers: Optional[Sequence[str]] = None,
        max_workers: Optional[int] = None,
    ) -> TableData:
        """Generate fake data as tabular data.

        Args:
            providers:
                List of provider names to generate a tabular data.
            rows:
                Number of rows in the tabular data.
            headers:
                List of header names.
            max_workers:
                Maximum number of workers to generate table data.
                In default, the same as the total number of CPUs.

        Returns:
            tabledata.TableData: Generated fake tabular data.
        """

        self.__validate_provider(providers)

        if rows < 0:
            raise ValueError("invalid rows")

        return TableData(
            table_name,
            headers if headers else providers,
            [
                [getattr(self.__fake, faker_name)() for faker_name in providers]
                for _row in range(rows)
            ],
            max_workers=max_workers,
        )

    @staticmethod
    def __validate_provider(providers: Sequence[str]) -> None:
        if not providers:
            raise ValueError("require provider(s)")

        diff = set(providers) - get_providers()

        if diff:
            raise ValueError(f"unknown providers found: {diff}")



import pytest
from pytablewriter import dumps_tabledata
from tabledata import TableData

from tblfaker import TableFaker

from ._common import print_test_result


dump_opts = {"line_break_handling": "escape"}


class Test_TableFaker_generate:
    @pytest.mark.parametrize(
        ["providers", "rows", "table_name", "expected"],
        [
            [
                ("name", "address"),
                2,
                None,
                TableData(
                    None,
                    ("name", "address"),
                    [
                        ("Ryan Gallagher", "6317 Mary Light\nSmithview, HI 13900"),
                        ("Amanda Johnson", "3608 Samuel Mews Apt. 337\nHousebury, WA 13608"),
                    ],
                ),
            ],
            [
                ("name", "address"),
                2,
                "test",
                TableData(
                    "test",
                    ("name", "address"),
                    [
                        ("Ryan Gallagher", "6317 Mary Light\nSmithview, HI 13900"),
                        ("Amanda Johnson", "3608 Samuel Mews Apt. 337\nHousebury, WA 13608"),
                    ],
                ),
            ],
            [
                ("name", "address"),
                4,
                None,
                TableData(
                    None,
                    ("name", "address"),
                    [
                        ("Ryan Gallagher", "6317 Mary Light\nSmithview, HI 13900"),
                        ("Amanda Johnson", "3608 Samuel Mews Apt. 337\nHousebury, WA 13608"),
                        ("Willie Heath", "868 Santiago Grove\nNicolehaven, NJ 05026"),
                        (
                            "Dr. Jared Ortega",
                            "517 Rodriguez Divide Suite 570\nWest Melinda, NH 85325",
                        ),
                    ],
                ),
            ],
        ],
    )
    def test_normal(self, providers, rows, table_name, expected):
        faker = TableFaker(seed=1)
        out = faker.generate(providers, rows, table_name=table_name)
        out_table = dumps_tabledata(out, **dump_opts)
        expected_table = dumps_tabledata(expected, **dump_opts)

        print_test_result(expected=expected_table, actual=out_table)

        assert out.table_name == expected.table_name
        assert out.headers == expected.headers
        assert out.num_rows == expected.num_rows

    def test_normal_specify_header_list(self):
        faker = TableFaker(seed=1)
        providers = ("file_name", "file_path")
        out = faker.generate(providers, 1, table_name="with headers", headers=("input", "output"))
        expected = TableData(
            "with headers", ("input", "output"), [("shake.wav", "/prepare/last.jpeg")]
        )
        out_table = dumps_tabledata(out, **dump_opts)
        expected_table = dumps_tabledata(expected, **dump_opts)

        print_test_result(expected=expected_table, actual=out_table)

        assert out.table_name == expected.table_name
        assert out.headers == expected.headers
        assert out.num_rows == expected.num_rows

    def test_normal_locale(self):
        faker = TableFaker(locale="ja_JP", seed=1)
        out = faker.generate(("name", "address"), rows=2)
        expected = TableData(
            None,
            ("name", "address"),
            [
                ("木村 陽一", "石川県香取市台東29丁目16番13号"),
                ("鈴木 晃", "兵庫県狛江市丹勢29丁目9番8号 南郷屋パーク008"),
            ],
        )
        out_table = dumps_tabledata(out, **dump_opts)
        expected_table = dumps_tabledata(expected, **dump_opts)

        print_test_result(expected=expected_table, actual=out_table)

        assert expected_table == out_table
        assert out == expected
        assert out != faker.generate(("name", "address"), rows=2)

    def test_normal_random(self):
        faker = TableFaker()
        providers = ("name", "address")
        rows = 16

        for _ in range(3):
            lhs = faker.generate(providers, rows)
            rhs = faker.generate(providers, rows)

            assert lhs.headers == rhs.headers
            assert lhs != rhs

    @pytest.mark.parametrize(
        ["providers", "rows", "expected"],
        [
            [["invalid provider"], 1, ValueError],
            [["name"], -1, ValueError],
        ],
    )
    def test_exception(self, providers, rows, expected):
        faker = TableFaker()

        with pytest.raises(expected):
            faker.generate(providers, rows)



import enum
import sys

import pytest
from faker import Factory

from tblfaker import get_locals, get_providers


class DummyEnum(enum.Enum):
    FOO = "foo"
    BAR = "bar"


def test_smoke_providers():
    fake = Factory.create()

    providers = get_providers()
    print(providers, file=sys.stderr)

    for provider in providers:
        if provider == "enum":
            getattr(fake, provider)(DummyEnum)
            continue

        getattr(fake, provider)()


def test_get_providers():
    expected = frozenset(
        [
            "address",
            "am_pm",
            "android_platform_token",
            "ascii_company_email",
            "ascii_email",
            "ascii_free_email",
            "ascii_safe_email",
            "bank_country",
            "bban",
            "binary",
            "boolean",
            "bothify",
            "bs",
            "building_number",
            "catch_phrase",
            "century",
            "chrome",
            "city",
            "city_prefix",
            "city_suffix",
            "color",
            "color_name",
            "company",
            "company_email",
            "company_suffix",
            "coordinate",
            "country",
            "country_calling_code",
            "country_code",
            "credit_card_expire",
            "credit_card_full",
            "credit_card_number",
            "credit_card_provider",
            "credit_card_security_code",
            "cryptocurrency",
            "cryptocurrency_code",
            "cryptocurrency_name",
            "csv",
            "currency",
            "currency_code",
            "currency_name",
            "currency_symbol",
            "date",
            "date_between",
            "date_between_dates",
            "date_object",
            "date_of_birth",
            "date_this_century",
            "date_this_decade",
            "date_this_month",
            "date_this_year",
            "date_time",
            "date_time_ad",
            "date_time_between",
            "date_time_between_dates",
            "date_time_this_century",
            "date_time_this_decade",
            "date_time_this_month",
            "date_time_this_year",
            "day_of_month",
            "dga",
            "day_of_week",
            "domain_name",
            "domain_word",
            "dsv",
            "ean",
            "ean13",
            "ean8",
            "ein",
            "email",
            "file_extension",
            "file_name",
            "file_path",
            "firefox",
            "first_name",
            "first_name_female",
            "first_name_male",
            "free_email",
            "free_email_domain",
            "future_date",
            "future_datetime",
            "hex_color",
            "hexify",
            "hostname",
            "http_method",
            "iban",
            "image_url",
            "internet_explorer",
            "invalid_ssn",
            "ios_platform_token",
            "ipv4",
            "ipv4_network_class",
            "ipv4_private",
            "ipv4_public",
            "ipv6",
            "isbn10",
            "isbn13",
            "iso8601",
            "itin",
            "job",
            "language_code",
            "last_name",
            "last_name_female",
            "last_name_male",
            "latitude",
            "latlng",
            "lexify",
            "license_plate",
            "linux_platform_token",
            "linux_processor",
            "local_latlng",
            "locale",
            "location_on_land",
            "longitude",
            "mac_address",
            "mac_platform_token",
            "mac_processor",
            "md5",
            "military_apo",
            "military_dpo",
            "military_ship",
            "military_state",
            "mime_type",
            "month",
            "month_name",
            "msisdn",
            "name",
            "name_female",
            "name_male",
            "null_boolean",
            "numerify",
            "opera",
            "paragraph",
            "paragraphs",
            "password",
            "past_date",
            "past_datetime",
            "phone_number",
            "port_number",
            "postalcode",
            "postalcode_in_state",
            "postalcode_plus4",
            "postcode",
            "postcode_in_state",
            "prefix",
            "prefix_female",
            "prefix_male",
            "profile",
            "psv",
            "pybool",
            "pydecimal",
            "pydict",
            "pyfloat",
            "pyint",
            "pyiterable",
            "pylist",
            "pyset",
            "pystr",
            "pystr_format",
            "pystruct",
            "pytuple",
            "random_choices",
            "random_digit",
            "random_digit_not_null",
            "random_digit_not_null_or_empty",
            "random_digit_or_empty",
            "random_element",
            "random_elements",
            "random_int",
            "random_letter",
            "random_letters",
            "random_lowercase_letter",
            "random_number",
            "random_sample",
            "random_uppercase_letter",
            "randomize_nb_elements",
            "rgb_color",
            "rgb_css_color",
            "safari",
            "safe_color_name",
            "safe_email",
            "safe_hex_color",
            "secondary_address",
            "seed_instance",
            "sentence",
            "sentences",
            "sha1",
            "sha256",
            "simple_profile",
            "slug",
            "ssn",
            "state",
            "state_abbr",
            "street_address",
            "street_name",
            "street_suffix",
            "suffix",
            "suffix_female",
            "suffix_male",
            "tar",
            "text",
            "texts",
            "time",
            "time_delta",
            "time_object",
            "time_series",
            "timezone",
            "tld",
            "tsv",
            "unix_device",
            "unix_partition",
            "unix_time",
            "upc_a",
            "upc_e",
            "uri",
            "uri_extension",
            "uri_page",
            "uri_path",
            "url",
            "user_agent",
            "user_name",
            "uuid4",
            "windows_platform_token",
            "word",
            "words",
            "year",
            "zip",
            "zipcode",
            "zipcode_in_state",
            "zipcode_plus4",
        ]
    )
    assert expected.issubset(get_providers())


@pytest.mark.parametrize(["value"], [[local] for local in get_locals()])
def test_smoke_locals(value):
    assert Factory.create(value).name()



from .__version__ import __author__, __copyright__, __email__, __license__, __version__
from ._common import get_locals, get_providers
from ._tblfaker import TableFaker


__all__ = (
    "__author__",
    "__copyright__",
    "__email__",
    "__license__",
    "__version__",
    "get_locals",
    "get_providers",
    "TableFaker",
)



import inspect
import re
from typing import AbstractSet, Final, Sequence

from faker import Factory


_non_provider_regexp: Final = re.compile("^(add|del|get|set)_[a-z_]+")
_non_provider_methods: Final = (
    "__delattr__",
    "__dir__",
    "__eq__",
    "__format__",
    "__ge__",
    "__getattribute__",
    "__gt__",
    "__hash__",
    "__init__",
    "__init_subclass__",
    "__le__",
    "__lt__",
    "__ne__",
    "__reduce__",
    "__reduce_ex__",
    "__repr__",
    "__setattr__",
    "__str__",
    "__subclasshook__",
    "_Generator__format_token",
    "format",
    "parse",
    "provider",
    "seed",
)


def _get_valid_providers() -> Sequence[str]:
    return tuple(
        method[0]
        for method in inspect.getmembers(Factory.create(), inspect.ismethod)
        if _non_provider_regexp.search(method[0]) is None and method[0] not in _non_provider_methods
    )


_valid_providers: Final = frozenset(_get_valid_providers())
_valid_locals: Final = frozenset(
    [
        "ar_EG",
        "ar_PS",
        "ar_SA",
        "bs_BA",
        "bg_BG",
        "cs_CZ",
        "de_DE",
        "dk_DK",
        "el_GR",
        "en_AU",
        "en_CA",
        "en_GB",
        "en_NZ",
        "en_US",
        "es_ES",
        "es_MX",
        "et_EE",
        "fa_IR",
        "fi_FI",
        "fr_FR",
        "hi_IN",
        "hr_HR",
        "hu_HU",
        "it_IT",
        "ja_JP",
        "ko_KR",
        "lt_LT",
        "lv_LV",
        "ne_NP",
        "nl_NL",
        "no_NO",
        "pl_PL",
        "pt_BR",
        "pt_PT",
        "ro_RO",
        "ru_RU",
        "sl_SI",
        "sv_SE",
        "tr_TR",
        "uk_UA",
        "zh_CN",
        "zh_TW",
        "ka_GE",
    ]
)


def get_providers() -> AbstractSet[str]:
    return _valid_providers


def get_locals() -> AbstractSet[str]:
    return _valid_locals
